import 'dart:developer';

import 'package:nudge_leaderboard_ui/features/leaderboard/data/datasources/createuser.dart';
import 'package:nudge_leaderboard_ui/features/leaderboard/data/models/create_user.dart';
import 'package:nudge_leaderboard_ui/features/leaderboard/presentation/widgets/leaderboard_nodata.dart';
import 'package:nudge_leaderboard_ui/features/leaderboard/presentation/widgets/leaderboard_scoredata.dart';

import '../../../../main.dart';
import '../../data/models/get_all_leaderboard_rewards.dart';
import '../../data/models/get_nudge_leaderboard.dart';
import '../widgets/appbar/leaderboard_appbar.dart';
import 'package:flutter/material.dart';

class LeaderboardScreen extends StatefulWidget {
  final String token;
  final String? rewardID;
  final String? externalID;
  final Map<String, dynamic>? props;

  const LeaderboardScreen({
    super.key,
    required this.token,
    this.rewardID,
    required this.externalID,
    this.props,
  });

  @override
  State<LeaderboardScreen> createState() =>
      // ignore: no_logic_in_create_state
      _LeaderboardScreenState(rewardID: rewardID);
}

class _LeaderboardScreenState extends State<LeaderboardScreen> {
  String? rewardID;
  bool isLoading = false;
  int currentPageNo = 1;
  int currentItemCount = 10;
  final int increment = 10;
  GetNudgeLeaderboard? nudgeLeaderboardData;
  GetAllLeaderboardRewards? allLeaderboardRewardsData;
  CreateUser? userData;

  DateTime now = DateTime.now();
  late int startDate;
  late int endDate;

  _LeaderboardScreenState({
    this.rewardID,
  });

  @override
  void initState() {
    super.initState();

    startDate = DateTime(now.year, now.month, 1).millisecondsSinceEpoch;
    endDate = DateTime.now().millisecondsSinceEpoch;

    log("startDate: $startDate, endDate: $endDate", name: "LeaderboardScreen");

    fetchRewardsData().then((_) {
      if (rewardID?.isEmpty == true) {
        // fetchLeaderboard();
        fetchTimeBasedLeaderboardOnScore();
      } else {
        // log('fetchLeaderboardOnReward', name: 'LeaderboardScreen');
        fetchLeaderboardOnReward(rewardID);
      }
    });
  }

  Future<void> updateDateRange(int startDate, int endDate) async {
    setState(() {
      this.startDate = startDate;
      this.endDate = endDate;
    });

    // await fetchTimeBasedLeaderboardOnScore();
    await fetchLeaderboardOnReward(rewardID);
  }

  Future<void> fetchRewardsData() async {
    try {
      CreateUser createUser = await createuserrequest(
        widget.token,
        widget.externalID,
        props: widget.props,
      );

      setState(() {
        userData = createUser;
      });
    } catch (error) {
      debugPrint('Error: $error');
    }
  }

  // Future<void> fetchLeaderboard() async {
  //   try {
  //     GetNudgeLeaderboard fetchedRewardsLeaderboard = await getNudgeLeaderboard(
  //       userData?.data.token,
  //       '$nudgeUrl/users/u/get/leaderboard/score',
  //     );
  //     setState(() {
  //       nudgeLeaderboardData = fetchedRewardsLeaderboard;
  //     });
  //   } catch (error) {
  //     debugPrint('Error: $error');
  //   }
  // }

  Future<void> fetchLeaderboardOnReward(String? rewardID) async {
    try {
      rewardID = '65ef5c8335d688a94be685a7';
      GetNudgeLeaderboard fetchedRewardsLeaderboard = await getNudgeLeaderboard(
        userData?.data.token,
        '$nudgeUrl/leaderboard/lb/get/reward/$rewardID?filter={"createdAt":{"\$gte":$startDate,"\$lte":$endDate}}&pageNo=$currentPageNo&noPerPage=$currentItemCount',
      );
      setState(() {
        nudgeLeaderboardData = fetchedRewardsLeaderboard;
      });
    } catch (error) {
      debugPrint('Error: $error');
    }
  }

  Future<void> fetchTimeBasedLeaderboardOnScore() async {
    try {
      GetNudgeLeaderboard fetchedRewardsLeaderboard = await getNudgeLeaderboard(
        userData?.data.token,
        '$nudgeUrl/leaderboard/lb/get/campaign/6608195d3126272c844a19bb?filter={"startDate":"$startDate","endDate":"$endDate"}&pageNo=$currentPageNo&noPerPage=$currentItemCount',
      );

      setState(() {
        nudgeLeaderboardData = fetchedRewardsLeaderboard;
      });
    } catch (error) {
      debugPrint('Error: $error');
    }
  }

  Future<void> loadMoreData() async {
    if (isLoading) {
      return;
    }
    setState(() {
      isLoading = true;
      currentPageNo = currentPageNo + 1;
    });

    await fetchLeaderboardOnReward(rewardID);
    // await fetchTimeBasedLeaderboardOnScore();

    if (currentItemCount + increment <=
        (nudgeLeaderboardData?.data.users.length ?? 0)) {
      currentItemCount += increment;
    } else {
      currentItemCount = (nudgeLeaderboardData?.data.users.length ?? 0);
    }
    setState(() {
      isLoading = false;
    });
  }

  @override
  Widget build(BuildContext context) {
    if (nudgeLeaderboardData == null) {
      return Scaffold(
        backgroundColor: const Color.fromRGBO(49, 162, 215, 1),
        appBar: LeaderboardAppBar(),
        body: Center(
          child: Column(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              _buildSubtitle("Loading..."),
              const SizedBox(height: 10),
              const CircularProgressIndicator(
                strokeWidth: 6,
                color: Colors.white,
              ),
            ],
          ),
        ),
      );
    } else if (nudgeLeaderboardData?.data.users.isEmpty ?? true) {
      return LeaderboardNoData();
    } else {
      return LeaderboardDataScore(
        nudgeLeaderboardData: nudgeLeaderboardData,
        currentItemCount: currentItemCount,
        currentPageNo: currentPageNo,
        loadMoreData: loadMoreData,
        refreshDataOnDateChanged: updateDateRange,
      );
    }
  }
}

Widget _buildSubtitle(String title) {
  return Container(
    padding: const EdgeInsets.only(
      top: 24,
      bottom: 16,
    ),
    child: Text(
      title,
      style: const TextStyle(
        fontSize: 20,
        color: Colors.white,
      ),
    ),
  );
}
