import 'package:nudge_leaderboard_ui/features/leaderboard/presentation/pages/leaderboard_page.dart';
import 'package:flutter/material.dart';
import 'package:nudge_leaderboard_ui/main.dart';
import 'package:dio/dio.dart' as dio;
import 'dart:convert';

class DropdownMenuExample extends StatefulWidget {
  const DropdownMenuExample({super.key});

  @override
  State<DropdownMenuExample> createState() => _DropdownMenuExampleState();
}

List<String> list = [
  "Production",
  "Staging",
];

class _DropdownMenuExampleState extends State<DropdownMenuExample> {
  String dropdownValue = list.first;

  @override
  Widget build(BuildContext context) {
    return DropdownMenu<String>(
        initialSelection: list.first,
        onSelected: (String? value) {
          if (value == "Staging") {
            nudgeUrl = "https://staging.api.nudgenow.com/api/v1";
            apiUrl = "https://dev.dashboard.api.nudgenow.com";
          }

          setState(() {
            dropdownValue = value!;
          });
        },
        dropdownMenuEntries:
            list.map<DropdownMenuEntry<String>>((String value) {
          return DropdownMenuEntry<String>(
              value: value,
              label: value,
              style: ButtonStyle(
                  backgroundColor:
                      MaterialStateProperty.all<Color>(Colors.white70)));
        }).toList(),
        textStyle: const TextStyle(color: Colors.white70),
        menuStyle: MenuStyle(
          backgroundColor: MaterialStateProperty.resolveWith<Color?>(
            (Set<MaterialState> states) {
              if (states.contains(MaterialState.focused)) {
                return Theme.of(context).colorScheme.primary.withOpacity(0.5);
              }
              return null; // Use the component's default.
            },
          ),
        ));
  }
}

class LoginPage extends StatefulWidget {
  const LoginPage({super.key});

  static String token = "";
  static String spinTheWheelId = "";

  @override
  State<LoginPage> createState() => _LoginPageState();
}

class _LoginPageState extends State<LoginPage> {
  TextEditingController nameController = TextEditingController();
  TextEditingController emailController = TextEditingController();
  TextEditingController rewardIdController = TextEditingController();
  TextEditingController phoneNumberController = TextEditingController();
  TextEditingController passwordController = TextEditingController();

  TextEditingController eventController = TextEditingController();
  TextEditingController statusController = TextEditingController();
  TextEditingController subEventController = TextEditingController();

  void initState() {
    super.initState();
    nameController.text = "Nudge Demo";
    emailController.text = "demo@nudgenow.com";
    rewardIdController.text = "";
    statusController.text = "";
    phoneNumberController.text =
        DateTime.now().millisecondsSinceEpoch.toString();
    eventController.text = "spinner_show";
    subEventController.text = "spinner_check";
  }

  Future<void> track(
      {required String event, String? subEvent, required String token}) async {
    var headers = {
      'authorization': MyAppState.token,
      'Content-Type': 'application/json'
    };

    var dataSubEvent = json.encode({
      "sessionId": "6y08052-198a-21we-a03c-75a7b87",
      "type": subEvent,
    });

    var dataEvent = json
        .encode({"sessionId": "6y08052-198a-21we-a03c-75a7b87", "type": event});

    if (subEvent != null) {
      await dio.Dio().request(
        '$nudgeUrl/events/e/track',
        options: dio.Options(
          method: 'POST',
          headers: headers,
        ),
        data: dataSubEvent,
      );
    }
    var response = await dio.Dio().request(
      '$nudgeUrl/events/e/track',
      options: dio.Options(
        method: 'POST',
        headers: headers,
      ),
      data: dataEvent,
    );

    print(response.statusCode);

    if (response.statusCode == 200 ||
        response.statusCode == 201 ||
        response.statusCode == 204) {
      statusController.text = statusController.text +
          "\n\n'${eventController.text}' event tracked at ${DateTime.now()}";
    }
  }

  Future<void> postRequest(
      String name, String password, String phoneNumber, String email,
      {required String event, String? subEvent}) async {
    var xheaders = {'Content-Type': 'application/json'};
    var xdata = json.encode({"email": email, "password": password});
    var d = dio.Dio();
    try {
      var xresponse = await d.request(
        '$apiUrl/clients/login',
        options: dio.Options(
          method: 'POST',
          headers: xheaders,
        ),
        data: xdata,
      );

      if (xresponse.statusCode == 200) {
        Map<String, dynamic> yd = xresponse.data;

        var pheaders = {
          'Authorization': 'Bearer ${yd["token"]}',
        };
        var d = dio.Dio();
        try {
          var yresponse = await d.request(
            '$apiUrl/clients/${yd["clientId"]}',
            options: dio.Options(
              method: 'GET',
              headers: pheaders,
            ),
          );

          if (yresponse.statusCode == 200) {
            Map<String, dynamic> zd = yresponse.data;

            var headers = {
              'authorization': zd["data"]["secretKey"],
              'Content-Type': 'application/json'
            };
            var data = json.encode({
              "name": name,
              "email": email,
              "externalId": phoneNumber,
            });
            var request = dio.Dio();

            try {
              var response = await request.request(
                '$nudgeUrl/users/u/create',
                options: dio.Options(
                  method: 'POST',
                  headers: headers,
                ),
                data: data,
              );

              if (response.statusCode == 201) {
                MyAppState.token = response.data["data"]["token"];

                print(MyAppState.token);

                Navigator.of(context).push(MaterialPageRoute(
                    builder: (context) => Scaffold(
                        appBar: AppBar(),
                        body: Container(
                            height: MediaQuery.of(context).size.height,
                            color: Colors.deepPurple,
                            child: Center(
                                child: Column(
                                    mainAxisAlignment: MainAxisAlignment.center,
                                    crossAxisAlignment:
                                        CrossAxisAlignment.center,
                                    children: [
                                  Padding(
                                    padding: const EdgeInsets.symmetric(
                                        horizontal: 20, vertical: 10),
                                    child: TextField(
                                      controller: eventController,
                                      style:
                                          const TextStyle(color: Colors.white),
                                      decoration: InputDecoration(
                                        hintText: 'Track an event',
                                        hintStyle: const TextStyle(
                                            color: Colors.white60),
                                        filled: true,
                                        fillColor:
                                            Colors.black.withOpacity(0.5),
                                        border: OutlineInputBorder(
                                          borderRadius:
                                              BorderRadius.circular(10.0),
                                        ),
                                      ),
                                    ),
                                  ),
                                  Padding(
                                    padding: const EdgeInsets.symmetric(
                                        horizontal: 20, vertical: 10),
                                    child: ElevatedButton(
                                        child: Text('Track Event'),
                                        onPressed: () {
                                          print(MyAppState.token);
                                          track(
                                              event: eventController.text,
                                              token: MyAppState.token);
                                        }),
                                  ),
                                  ElevatedButton(
                                      onPressed: () {
                                        Navigator.of(context).push(
                                          MaterialPageRoute(
                                              builder: (context) =>
                                                  LeaderboardScreen(
                                                    token: zd["data"]
                                                        ["secretKey"],
                                                    externalID:
                                                        phoneNumberController
                                                            .text,
                                                    rewardID:
                                                        rewardIdController.text,
                                                  )),
                                        );
                                      },
                                      child: Text('Show Leaderboard')),
                                  Container(
                                      padding: EdgeInsets.all(20),
                                      height:
                                          MediaQuery.of(context).size.height *
                                              0.4,
                                      width: MediaQuery.of(context).size.width *
                                          0.8,
                                      color: Colors.black26,
                                      child: Text(statusController.text))
                                ]))))));
              } else {
                Navigator.of(context).push(MaterialPageRoute(
                  builder: (context) => const NoDataPage(),
                ));
              }
            } catch (e, st) {
              print(e);
              print(st);
              Navigator.of(context).push(MaterialPageRoute(
                builder: (context) => NoDataPage(
                    text: "ERROR in Init Session API \n" + e.toString()),
              ));
            }
          }
        } catch (e, st) {
          print(e);
          print(st);
          Navigator.of(context).push(MaterialPageRoute(
            builder: (context) =>
                NoDataPage(text: "ERROR in User Create API \n" + e.toString()),
          ));
        }
      } else {
        Navigator.of(context).push(MaterialPageRoute(
          builder: (context) => const NoDataPage(),
        ));
      }
    } catch (e, st) {
      print(e);
      print(st);
      Navigator.of(context).push(MaterialPageRoute(
        builder: (context) =>
            NoDataPage(text: "ERROR in Login API \n" + e.toString()),
      ));
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      extendBodyBehindAppBar: true,
      appBar: AppBarSpinTheWheel(),
      bottomNavigationBar: BottomNavBarSingle(
          text: 'Start',
          onClick: () {
            FocusManager.instance.primaryFocus?.unfocus();
            postRequest(
              nameController.text,
              passwordController.text,
              phoneNumberController.text,
              emailController.text,
              event: eventController.text,
              subEvent: subEventController.text == ""
                  ? null
                  : subEventController.text,
            );
          }),
      backgroundColor: const Color.fromRGBO(51, 0, 120, 1),
      body: SingleChildScrollView(
        child: Container(
          width: double.infinity,
          height: MediaQuery.of(context).size.height,
          decoration: const BoxDecoration(
            color: Colors.deepPurple,
          ),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.center,
            children: [
              const SizedBox(height: 150),
              const Padding(
                padding: EdgeInsets.symmetric(horizontal: 20, vertical: 10),
                child: DropdownMenuExample(),
              ),
              Padding(
                padding:
                    const EdgeInsets.symmetric(horizontal: 20, vertical: 10),
                child: TextField(
                  controller: nameController,
                  style: const TextStyle(color: Colors.white),
                  decoration: InputDecoration(
                    hintText: 'Name',
                    hintStyle: const TextStyle(color: Colors.white60),
                    filled: true,
                    fillColor: Colors.black.withOpacity(0.5),
                    border: OutlineInputBorder(
                      borderRadius: BorderRadius.circular(10.0),
                    ),
                  ),
                ),
              ),
              Padding(
                padding:
                    const EdgeInsets.symmetric(horizontal: 20, vertical: 10),
                child: TextField(
                  controller: emailController,
                  style: const TextStyle(color: Colors.white),
                  decoration: InputDecoration(
                    hintText: 'Email',
                    hintStyle: const TextStyle(color: Colors.white60),
                    filled: true,
                    fillColor: Colors.black.withOpacity(0.5),
                    border: OutlineInputBorder(
                      borderRadius: BorderRadius.circular(10.0),
                    ),
                  ),
                ),
              ),
              Padding(
                padding:
                    const EdgeInsets.symmetric(horizontal: 20, vertical: 10),
                child: TextField(
                  obscureText: true,
                  controller: passwordController,
                  style: const TextStyle(color: Colors.white),
                  decoration: InputDecoration(
                    hintText: 'Password',
                    hintStyle: const TextStyle(color: Colors.white60),
                    filled: true,
                    fillColor: Colors.black.withOpacity(0.5),
                    border: OutlineInputBorder(
                      borderRadius: BorderRadius.circular(10.0),
                    ),
                  ),
                ),
              ),
              Padding(
                padding:
                    const EdgeInsets.symmetric(horizontal: 20, vertical: 10),
                child: TextField(
                  controller: phoneNumberController,
                  style: const TextStyle(color: Colors.white),
                  decoration: InputDecoration(
                    hintText: 'External ID',
                    hintStyle: const TextStyle(color: Colors.white60),
                    filled: true,
                    fillColor: Colors.black.withOpacity(0.5),
                    border: OutlineInputBorder(
                      borderRadius: BorderRadius.circular(10.0),
                    ),
                  ),
                ),
              ),
              Padding(
                padding:
                    const EdgeInsets.symmetric(horizontal: 20, vertical: 10),
                child: TextField(
                  controller: rewardIdController,
                  style: const TextStyle(color: Colors.white),
                  decoration: InputDecoration(
                    hintText: 'RewardId',
                    hintStyle: const TextStyle(color: Colors.white60),
                    filled: true,
                    fillColor: Colors.black.withOpacity(0.5),
                    border: OutlineInputBorder(
                      borderRadius: BorderRadius.circular(10.0),
                    ),
                  ),
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }
}

class BottomNavBarSingle extends StatefulWidget {
  final String text;
  final Function onClick;
  final bool disabled;

  const BottomNavBarSingle(
      {super.key,
      required this.text,
      required this.onClick,
      this.disabled = false});

  @override
  State<BottomNavBarSingle> createState() => _BottomNavBarSingleState();
}

class _BottomNavBarSingleState extends State<BottomNavBarSingle> {
  @override
  Widget build(BuildContext context) {
    return BottomAppBar(
      color: const Color.fromRGBO(50, 0, 120, 1),
      child: SizedBox(
        height: 74.0, // Set the height of the bar
        child: (widget.disabled)
            ? Container()
            : InkWell(
                onTap: () {
                  setState(() {
                    widget.onClick();
                  });
                },
                child: Padding(
                  padding: const EdgeInsets.symmetric(
                      vertical: 12.0, horizontal: 16),
                  child: Container(
                    width: 344,
                    height: 56,
                    decoration: BoxDecoration(
                        color: const Color.fromRGBO(255, 0, 177, 1),
                        borderRadius: BorderRadius.circular(32)),
                    child: Align(
                      alignment: Alignment.center,
                      child: Padding(
                        padding: const EdgeInsets.all(10.0),
                        child: Text(
                          widget.text,
                          style: const TextStyle(
                            fontSize: 14,
                            fontWeight: FontWeight.w500,
                            color: Colors.white, // Text color
                          ),
                        ),
                      ),
                    ),
                  ),
                ),
              ),
      ),
    );
  }
}

class NoDataPage extends StatefulWidget {
  const NoDataPage(
      {super.key, this.text = "NO SPIN THE WHEEL AVAILABLE CURRENTLY"});

  final String text;

  @override
  State<NoDataPage> createState() => _NoDataPageState();
}

class AppBarSpinTheWheel extends StatelessWidget
    implements PreferredSizeWidget {
  @override
  Size get preferredSize => const Size.fromHeight(kToolbarHeight);

  @override
  Widget build(BuildContext context) {
    return AppBar(
      backgroundColor: Colors.transparent,
      elevation: 0,
      leading: IconButton(
        icon: Icon(Icons.arrow_back),
        onPressed: () {
          Navigator.pop(context);
        },
      ),
      actions: <Widget>[],
    );
  }
}

class _NoDataPageState extends State<NoDataPage> {
  @override
  void dispose() {
    super.dispose();
  }

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      extendBodyBehindAppBar: true,
      appBar: AppBarSpinTheWheel(),
      bottomNavigationBar: BottomNavBarSingle(
        text: 'Back to Home',
        onClick: () {
          Navigator.pop(context);
        },
      ),
      backgroundColor: const Color.fromRGBO(51, 0, 120, 1),
      body: SingleChildScrollView(
        child: Container(
          width: double.infinity,
          height: MediaQuery.of(context).size.height,
          decoration: const BoxDecoration(
            color: Colors.deepPurple,
          ),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.center,
            children: [
              Container(
                padding: const EdgeInsets.all(30),
                height: MediaQuery.of(context).size.height * 0.55,
                child: Center(
                  child: Text(
                    widget.text,
                    textAlign: TextAlign.center,
                    style: TextStyle(
                      color: Colors.white,
                      fontSize: 36,
                      fontWeight: FontWeight.w800,
                    ),
                  ),
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }
}
