import 'package:flutter/material.dart';
import 'package:nudge_leaderboard_ui/extras/cached_network_image/cached_network_image.dart';

class RankContainer extends StatelessWidget {
  final int points;
  final String name;
  final String? photoUrl;
  final int rank;

  const RankContainer({
    super.key,
    required this.points,
    required this.name,
    required this.rank,
    this.photoUrl,
  });

  String formatNumbers(int number) {
    if (number >= 1000 && number < 1000000) {
      return '${(number / 1000).toStringAsFixed(1)}K';
    } else if (number >= 1000000 && number < 1000000000) {
      return '${(number / 1000000).toStringAsFixed(1)}M';
    } else if (number >= 1000000000) {
      return '${(number / 1000000000).toStringAsFixed(1)}B';
    } else {
      return number.toString();
    }
  }

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.symmetric(
        vertical: 4.0,
        horizontal: 8,
      ),
      child: Container(
        decoration: BoxDecoration(
          color: Colors.white,
          borderRadius: BorderRadius.circular(10),
          border: Border.all(
            color: const Color.fromRGBO(
              230,
              233,
              237,
              1,
            ),
            width: 2.0,
          ),
        ),
        child: Padding(
          padding: const EdgeInsets.all(4.0),
          child: Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              Expanded(
                flex: 1,
                child: Container(
                  padding: const EdgeInsets.only(
                    left: 10.0,
                  ),
                  width: MediaQuery.of(context).size.width * 0.1,
                  child: RoundContainerWithNumber(
                    rank: rank,
                  ),
                ),
              ),
              const SizedBox(
                width: 15,
              ),
              Expanded(
                flex: 3,
                child: Container(
                  // width: MediaQuery.of(context).size.width * 0.55,
                  padding: const EdgeInsets.only(left: 15.0),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      AvatarContainer(
                        name: name,
                        photoUrl: photoUrl,
                      ),
                    ],
                  ),
                ),
              ),
              Expanded(
                flex: 1,
                child: Text(
                  formatNumbers(points),
                  style: const TextStyle(
                    color: Color.fromRGBO(126, 140, 160, 1),
                    fontSize: 16,
                  ),
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }
}

class RoundContainerWithNumber extends StatelessWidget {
  final int rank;

  const RoundContainerWithNumber({
    super.key,
    required this.rank,
  });

  @override
  Widget build(BuildContext context) {
    return Container(
      width: 43.0, // You can adjust the width and height as needed.
      height: 43.0,
      decoration: BoxDecoration(
        color: Colors.white,
        border: Border.all(
          color: const Color.fromRGBO(208, 213, 221, 1),
          width: 2.0, // You can adjust the border width as needed.
        ),
        shape: BoxShape.circle,
      ),
      child: Center(
        child: Text(
          rank > 1000000
              ? ("${(rank ~/ 1000000)}M+")
              : rank > 1000
                  ? ("${(rank ~/ 1000)}K+")
                  : '$rank',
          style: const TextStyle(
            fontSize: 14.0, // You can adjust the font size as needed.
            color: Colors.black,
          ),
        ),
      ),
    );
  }
}

class AvatarContainer extends StatelessWidget {
  final String name;
  final String? photoUrl;

  const AvatarContainer({
    Key? key, // Fix the super.key parameter
    required this.name,
    this.photoUrl,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      width: MediaQuery.of(context).size.width * 0.6,
      child: Row(
        mainAxisAlignment: MainAxisAlignment.start,
        children: [
          CircularAvatarContainer(
            photoUrl: photoUrl,
          ),
          const SizedBox(
            width: 10,
          ),
          Expanded(
            child: Text(
              name,
              style: const TextStyle(
                color: Colors.black,
                fontSize: 16,
                fontWeight: FontWeight.w600,
              ),
              overflow: TextOverflow.ellipsis,
            ),
          ),
        ],
      ),
    );
  }
}

class CircularAvatarContainer extends StatelessWidget {
  final String? photoUrl;

  const CircularAvatarContainer({
    super.key,
    this.photoUrl,
  });

  @override
  Widget build(BuildContext context) {
    return Container(
        width: 50.0,
        height: 50.0,
        decoration: BoxDecoration(
          shape: BoxShape.circle,
          border: Border.all(
            color: Colors.blue,
            width: 2.0,
          ),
        ),
        child: CircleAvatar(
            backgroundColor: Colors.grey,
            radius: 20.0,
            backgroundImage: CachedNetworkImageProvider(photoUrl ??
                'https://upload.wikimedia.org/wikipedia/commons/thumb/b/b5/Windows_10_Default_Profile_Picture.svg/240px-Windows_10_Default_Profile_Picture.svg.png')));
  }
}
